package org.apache.bcel.classfile;

import java.io.DataInput;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Arrays;

public final class Module extends Attribute {
   private final int moduleNameIndex;
   private final int moduleFlags;
   private final int moduleVersionIndex;
   private ModuleRequires[] requiresTable;
   private ModuleExports[] exportsTable;
   private ModuleOpens[] opensTable;
   private final int usesCount;
   private final int[] usesIndex;
   private ModuleProvides[] providesTable;

   Module(int nameIndex, int length, DataInput input, ConstantPool constantPool) throws IOException {
      super((byte)22, nameIndex, length, constantPool);
      this.moduleNameIndex = input.readUnsignedShort();
      this.moduleFlags = input.readUnsignedShort();
      this.moduleVersionIndex = input.readUnsignedShort();
      int requiresCount = input.readUnsignedShort();
      this.requiresTable = new ModuleRequires[requiresCount];

      int exportsCount;
      for(exportsCount = 0; exportsCount < requiresCount; ++exportsCount) {
         this.requiresTable[exportsCount] = new ModuleRequires(input);
      }

      exportsCount = input.readUnsignedShort();
      this.exportsTable = new ModuleExports[exportsCount];

      int opensCount;
      for(opensCount = 0; opensCount < exportsCount; ++opensCount) {
         this.exportsTable[opensCount] = new ModuleExports(input);
      }

      opensCount = input.readUnsignedShort();
      this.opensTable = new ModuleOpens[opensCount];

      int providesCount;
      for(providesCount = 0; providesCount < opensCount; ++providesCount) {
         this.opensTable[providesCount] = new ModuleOpens(input);
      }

      this.usesCount = input.readUnsignedShort();
      this.usesIndex = new int[this.usesCount];

      for(providesCount = 0; providesCount < this.usesCount; ++providesCount) {
         this.usesIndex[providesCount] = input.readUnsignedShort();
      }

      providesCount = input.readUnsignedShort();
      this.providesTable = new ModuleProvides[providesCount];

      for(int i = 0; i < providesCount; ++i) {
         this.providesTable[i] = new ModuleProvides(input);
      }

   }

   public void accept(Visitor v) {
      v.visitModule(this);
   }

   public Attribute copy(ConstantPool constantPool) {
      Module c = (Module)this.clone();
      c.requiresTable = new ModuleRequires[this.requiresTable.length];
      Arrays.setAll(c.requiresTable, (i) -> {
         return this.requiresTable[i].copy();
      });
      c.exportsTable = new ModuleExports[this.exportsTable.length];
      Arrays.setAll(c.exportsTable, (i) -> {
         return this.exportsTable[i].copy();
      });
      c.opensTable = new ModuleOpens[this.opensTable.length];
      Arrays.setAll(c.opensTable, (i) -> {
         return this.opensTable[i].copy();
      });
      c.providesTable = new ModuleProvides[this.providesTable.length];
      Arrays.setAll(c.providesTable, (i) -> {
         return this.providesTable[i].copy();
      });
      c.setConstantPool(constantPool);
      return c;
   }

   public void dump(DataOutputStream file) throws IOException {
      super.dump(file);
      file.writeShort(this.moduleNameIndex);
      file.writeShort(this.moduleFlags);
      file.writeShort(this.moduleVersionIndex);
      file.writeShort(this.requiresTable.length);
      ModuleRequires[] var2 = this.requiresTable;
      int var3 = var2.length;

      int var4;
      for(var4 = 0; var4 < var3; ++var4) {
         ModuleRequires entry = var2[var4];
         entry.dump(file);
      }

      file.writeShort(this.exportsTable.length);
      ModuleExports[] var6 = this.exportsTable;
      var3 = var6.length;

      for(var4 = 0; var4 < var3; ++var4) {
         ModuleExports entry = var6[var4];
         entry.dump(file);
      }

      file.writeShort(this.opensTable.length);
      ModuleOpens[] var7 = this.opensTable;
      var3 = var7.length;

      for(var4 = 0; var4 < var3; ++var4) {
         ModuleOpens entry = var7[var4];
         entry.dump(file);
      }

      file.writeShort(this.usesIndex.length);
      int[] var8 = this.usesIndex;
      var3 = var8.length;

      for(var4 = 0; var4 < var3; ++var4) {
         int entry = var8[var4];
         file.writeShort(entry);
      }

      file.writeShort(this.providesTable.length);
      ModuleProvides[] var9 = this.providesTable;
      var3 = var9.length;

      for(var4 = 0; var4 < var3; ++var4) {
         ModuleProvides entry = var9[var4];
         entry.dump(file);
      }

   }

   public ModuleExports[] getExportsTable() {
      return this.exportsTable;
   }

   public ModuleOpens[] getOpensTable() {
      return this.opensTable;
   }

   public ModuleProvides[] getProvidesTable() {
      return this.providesTable;
   }

   public ModuleRequires[] getRequiresTable() {
      return this.requiresTable;
   }

   public String toString() {
      ConstantPool cp = super.getConstantPool();
      StringBuilder buf = new StringBuilder();
      buf.append("Module:\n");
      buf.append("  name:    ").append(Utility.pathToPackage(cp.getConstantString(this.moduleNameIndex, (byte)19))).append("\n");
      buf.append("  flags:   ").append(String.format("%04x", this.moduleFlags)).append("\n");
      String version = this.moduleVersionIndex == 0 ? "0" : cp.getConstantString(this.moduleVersionIndex, (byte)1);
      buf.append("  version: ").append(version).append("\n");
      buf.append("  requires(").append(this.requiresTable.length).append("):\n");
      ModuleRequires[] var4 = this.requiresTable;
      int var5 = var4.length;

      int var6;
      for(var6 = 0; var6 < var5; ++var6) {
         ModuleRequires module = var4[var6];
         buf.append("    ").append(module.toString(cp)).append("\n");
      }

      buf.append("  exports(").append(this.exportsTable.length).append("):\n");
      ModuleExports[] var9 = this.exportsTable;
      var5 = var9.length;

      for(var6 = 0; var6 < var5; ++var6) {
         ModuleExports module = var9[var6];
         buf.append("    ").append(module.toString(cp)).append("\n");
      }

      buf.append("  opens(").append(this.opensTable.length).append("):\n");
      ModuleOpens[] var10 = this.opensTable;
      var5 = var10.length;

      for(var6 = 0; var6 < var5; ++var6) {
         ModuleOpens module = var10[var6];
         buf.append("    ").append(module.toString(cp)).append("\n");
      }

      buf.append("  uses(").append(this.usesIndex.length).append("):\n");
      int[] var11 = this.usesIndex;
      var5 = var11.length;

      for(var6 = 0; var6 < var5; ++var6) {
         int index = var11[var6];
         String className = cp.getConstantString(index, (byte)7);
         buf.append("    ").append(Utility.compactClassName(className, false)).append("\n");
      }

      buf.append("  provides(").append(this.providesTable.length).append("):\n");
      ModuleProvides[] var12 = this.providesTable;
      var5 = var12.length;

      for(var6 = 0; var6 < var5; ++var6) {
         ModuleProvides module = var12[var6];
         buf.append("    ").append(module.toString(cp)).append("\n");
      }

      return buf.substring(0, buf.length() - 1);
   }
}
